@@ -17,7 +17,7 @@
 ################################################################################
 
 from pyflink.java_gateway import get_gateway
-from pyflink.table.types import _to_java_type
+from pyflink.table.types import _to_java_data_type
 from pyflink.util import java_utils
 
 __all__ = ['TableSink', 'CsvTableSink', 'WriteMode']
@@ -68,11 +68,11 @@ class CsvTableSink(TableSink):
             j_write_mode = None
         else:
             raise Exception('Unsupported write_mode: %s' % write_mode)
-        j_csv_table_sink = gateway.jvm.CsvTableSink(
-            path, field_delimiter, num_files, j_write_mode)
         j_field_names = java_utils.to_jarray(gateway.jvm.String, field_names)
         j_field_types = java_utils.to_jarray(
-            gateway.jvm.TypeInformation,
-            [_to_java_type(field_type) for field_type in field_types])
-        j_csv_table_sink = j_csv_table_sink.configure(j_field_names, j_field_types)
+            gateway.jvm.DataType,
+            [_to_java_data_type(field_type) for field_type in field_types])
+        j_csv_table_sink = gateway.jvm.CsvTableSink(
+            path, field_delimiter, num_files, j_write_mode, j_field_names, j_field_types)
+
         super(CsvTableSink, self).__init__(j_csv_table_sink)
